/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */

/* This file was modified liberally to make it work for Rsync and create
   a scoreboard for access by external programs.
*/

#include "rsync.h"
//#include "scoreboard.h"
#include "nwreg.h"
 

scoreboard *ap_scoreboard_image = NULL;
const char *ap_scoreboard_fname = NULL;
int ap_extended_status = 0;

struct ap_sb_handle_t {
    int child_num;
    int thread_num;
};

static int server_limit, thread_limit;
static size_t scoreboard_size;

#ifndef SUCCESS
#define SUCCESS 0
#endif

#ifndef FAIL
#define FAIL -1
#endif

struct reg_stats {
	int	status;						// In progress, or some completion code from rsync
	int64 total_size;
	int64 total_transferred_size;
	int64 total_written;
	int64 total_read;
	int64 literal_data;
	int64 matched_data;
	int64 flist_size;
	int64 flist_index;
	int64 num_files;
	int64 num_transferred_files;
	char	error[SCOREMAXPATHLEN*2];
};

#define ClearScreen()           clrscr()

#define ADMIN "ADMIN"
#define DATA  "DATA"

#undef start_time

static long ROUND(double value)
{
	long intpart;

	intpart = (long)value;
	value = value - intpart;
	if (value >= 0.5) intpart++;
	
	return intpart;
}

int ap_calc_scoreboard_size(void)
{
	if (IsBOMA() == SUCCESS)
		thread_limit = 2;
	else
		thread_limit = 100;
		
	server_limit = 1;
//    ap_mpm_query(AP_MPMQ_HARD_LIMIT_THREADS, &thread_limit);
  //  ap_mpm_query(AP_MPMQ_HARD_LIMIT_DAEMONS, &server_limit);
    scoreboard_size = sizeof(global_score);
    scoreboard_size += sizeof(process_score) * server_limit;
    scoreboard_size += sizeof(worker_score) * server_limit * thread_limit;
    return scoreboard_size;
}

void ap_init_scoreboard(void *shared_score)
{
    char *more_storage;
    int i;
    
    ap_calc_scoreboard_size();
    ap_scoreboard_image = 
        calloc(1, sizeof(scoreboard) + server_limit * sizeof(worker_score *));
//	rprintf(FLOG, "scoreboard %x\n", ap_scoreboard_image);
    more_storage = shared_score;
//	rprintf(FLOG, "scoreboard shared_score %x\n", shared_score);
    ap_scoreboard_image->global = (global_score *)more_storage;
    more_storage += sizeof(global_score);
    ap_scoreboard_image->parent = (process_score *)more_storage;
    more_storage += sizeof(process_score) * server_limit;
    ap_scoreboard_image->servers = 
        (worker_score **)((char*)ap_scoreboard_image + sizeof(scoreboard));
    for (i = 0; i < server_limit; i++) {
        ap_scoreboard_image->servers[i] = (worker_score *)more_storage;
        more_storage += thread_limit * sizeof(worker_score);
    }
    assert(more_storage == (char*)shared_score + scoreboard_size);
    ap_scoreboard_image->global->server_limit = server_limit;
    ap_scoreboard_image->global->thread_limit = thread_limit;
}

/* Create or reinit an existing scoreboard. The MPM can control whether
 * the scoreboard is shared across multiple processes or not
 */
int ap_create_scoreboard(ap_scoreboard_e sb_type)
{
	char ScreenName[128];
	char socketName[128];
	extern scr_t gScoreHandle;
	extern int m_server_fd;
    int running_gen = 0;
    int i;
	rtag_t screenTag;

#if APR_HAS_SHARED_MEMORY
    apr_status_t rv;
#endif
    if (ap_scoreboard_image) {
        running_gen = ap_scoreboard_image->global->running_generation;
        ap_scoreboard_image->global->restart_time = time(NULL);
        memset(ap_scoreboard_image->parent, 0, 
               sizeof(process_score) * server_limit);
        for (i = 0; i < server_limit; i++) {
            memset(ap_scoreboard_image->servers[i], 0,
                   sizeof(worker_score) * thread_limit);
        }
        return SB_OK;
    }

    ap_calc_scoreboard_size();
    {
        /* A simple malloc will suffice */
        void *sb_mem = calloc(1, scoreboard_size);
        if (sb_mem == NULL) {
            rprintf(FERROR,
                         "(%d)%s: cannot allocate scoreboard",
                         errno, strerror(errno));
            return RERR_MALLOC;
        }
        ap_init_scoreboard(sb_mem);
    }

    ap_scoreboard_image->global->sb_type = sb_type;
    ap_scoreboard_image->global->running_generation = running_gen;
    ap_scoreboard_image->global->restart_time = computeCurrentTime();

	if (sb_type == SB_DISPLAY)
	{
		if (0 == getRsyncSocketName(m_server_fd, socketName))
		{
		   strcpy(ScreenName, "Rsync ");
		   strcat(ScreenName, socketName);
		   strcat(ScreenName, " Scoreboard");
		}
		else
		{
			strcpy(ScreenName, RSYNC_DAEMON_SCREEN_NAME);
			strcat(ScreenName, " Scoreboard");
			rprintf(FLOG, "Unable to get the socket name for %d\n", m_server_fd);
		}

		screenTag = AllocateResourceTag(getnlmhandle(), "RsyncDaemonScreen", ScreenSignature);
		OpenScreen(ScreenName, screenTag, &gScoreHandle);
		if (gScoreHandle == NULL)
		{
			rprintf(FERROR, "rsyncd version %s starting, unable to create scoreboard screen %s\n",
			RSYNC_VERSION, RSYNC_DAEMON_SCREEN_NAME);
			exit_cleanup(RERR_SYNTAX);
		}
		ActivateScreen(gScoreHandle);
	}
	
//	consoleprintf( "ap_create_scoreboard %x %x\n", ap_scoreboard_image, getpid());
//	rprintf(FLOG, "scoreboard %x %x\n", ap_scoreboard_image, &ap_scoreboard_image->servers[0][0]);
//	for( y = 0; y < thread_limit; y++)
//	{
//		worker_score * ws = &ap_scoreboard_image->servers[0][y];
//		consoleprintf("worker %x %d\n", ws, ws->status);
//	}
	
    return SB_OK;
}

static int ap_cleanup_shared_mem(void *d)
{
#if APR_HAS_SHARED_MEMORY
    free(ap_scoreboard_image);
    ap_scoreboard_image = NULL;
    apr_shm_destroy(ap_scoreboard_shm);
#endif
    return SUCCESS;
}

int ap_cleanup_scoreboard(void *d)
{
//	consoleprintf( "ap_cleanup_scoreboard %x %d\n", ap_scoreboard_image, getpid());

    if (ap_scoreboard_image == NULL) {
        return SUCCESS;
    }
    if (ap_scoreboard_image->global->sb_type == SB_SHARED) {
        ap_cleanup_shared_mem(NULL);
    }
    else {
        free(ap_scoreboard_image->global);
        free(ap_scoreboard_image);
        ap_scoreboard_image = NULL;
    }
    return SUCCESS;
}

worker_score * ap_get_scoreboard_worker(int x, int y)
{
    if (((x < 0) || (server_limit < x)) ||
        ((y < 0) || (thread_limit < y))) {
        return(NULL); /* Out of range */
    }
    return &ap_scoreboard_image->servers[x][y];
}

process_score * ap_get_scoreboard_process(int x)
{
    if ((x < 0) || (server_limit < x)) {
        return(NULL); /* Out of range */
    }
    if (ap_scoreboard_image)
	    return &ap_scoreboard_image->parent[x];
	else 
		return NULL;
}

global_score * ap_get_scoreboard_global()
{
    return ap_scoreboard_image->global;
}

int ap_update_child_status_from_indexes(int child_num,
                                                    int thread_num,
                                                    int status)
{
    int old_status;
    worker_score *ws;
    process_score *ps;

//	rprintf(FLOG, "scoreboard %x %x\n", ap_scoreboard_image, &ap_scoreboard_image->servers[0][0]);
//	consoleprintf( "ap_update_child_status_from_indexes %d %d %d\n", child_num, thread_num, status);

    if (child_num < 0) {
        return -1;
    }

    ws = &ap_scoreboard_image->servers[child_num][thread_num];
    old_status = ws->status;
    ws->status = status;
	//consoleprintf( "ap_update_child_status_from_indexes %x %d\n", ws, ws->status);

    ps = &ap_scoreboard_image->parent[child_num];
    
    if (status == SERVER_READY
        && old_status == SERVER_STARTING) {
        ws->thread_num = child_num * thread_limit + thread_num;
        ps->generation = 0;
        ws->start_time = computeCurrentTime();
    }

    if (ap_extended_status) {
        ws->last_used = time(NULL);
        if (status == SERVER_READY || status == SERVER_DEAD) {
            /*
             * Reset individual counters
             */
            if (status == SERVER_DEAD) {
                ws->my_access_count = 0L;
                ws->my_bytes_served = 0L;
            }
            ws->conn_count = 0;
            ws->conn_bytes = 0;
        }
#if 0        
        if (r) {
            conn_rec *c = r->connection;
            strncpy(ws->client, ap_get_remote_host(c, r->per_dir_config,
                        REMOTE_NOLOOKUP, NULL), sizeof(ws->client));
            if (r->the_request == NULL) {
                strncpy(ws->request, "NULL", sizeof(ws->request));
            } else if (r->parsed_uri.password == NULL) {
                strncpy(ws->request, r->the_request, sizeof(ws->request));
            } else {
                /* Don't reveal the password in the server-status view */
                strncpy(ws->request, apr_pstrcat(r->pool, r->method, " ",
                            apr_uri_unparse(r->pool, &r->parsed_uri,
                            APR_URI_UNP_OMITPASSWORD),
                            r->assbackwards ? NULL : " ", r->protocol, NULL),
                            sizeof(ws->request));
            }
            strncpy(ws->vhost, r->server->server_hostname,
                        sizeof(ws->vhost));
        }
#endif        
    }
    
    return old_status;
}

#undef stats

void GetRsyncKey(char *buffer, int maxSize)
{
	extern int am_daemon;
	if (am_daemon)
	{
		sprintf(buffer, "%s", REGISTRYKEY_RSYNC_SYS);
//		consoleprintf( "GetRsyncKey %s\n", buffer);
	}
	else
	{
		sprintf(buffer, "%s", REGISTRYKEY_RSYNC);
//		consoleprintf( "GetRsyncKey %s\n", buffer);
	}
}

/******************************************************************************
 *
 * DESCRIPTION:
 *
 *
 *
 ****************************************************************************/
int GetSBTimeFromRegistry(char*pRegKey, long *pTime, char *name)
{
	long	lResult;
	HKEY	hkeyMyKey;
	char 	buf[100];
	long	lSize = sizeof(buf);
	double	status;
	long	lDisposition;
	char 	keybuf[200];
	char 	fullbuf[200];
	long 	Type;

	*pTime = 0;
	GetRsyncKey(keybuf, sizeof(keybuf));
	sprintf(fullbuf, "%s\\%s", keybuf, name);
	//consoleprintf( "GetTimeFromRegistry %s\n", fullbuf);

	// Create or open the registry key
	lResult = RegCreateKeyEx(
					HKEY_LOCAL_MACHINE, 	// Key
					(BYTE*)fullbuf, 			// Sub-key
					0,						// Reserved
					NULL,					// Reserved
					REG_OPTION_NON_VOLATILE,// Really save in registry
					KEY_ALL_ACCESS,			// Allow all access
					0,						// Reserved
					&hkeyMyKey,				// Returns this handle
					&lDisposition);			// Was it created or just opened?

	if (lResult != 0)
	{
		return FAIL;
	}

	// Write the registry entry
	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)pRegKey,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
			sprintf(buf,"0");
	}

	status = atof(buf);
	//consoleprintf("GetTime %s %f\n", buf, status);

	status = status / 1000.0;
	//consoleprintf("GetTime %s %f\n", buf, status);

	*pTime = (long)status;
	// Close the registry
	lResult = RegCloseKey(hkeyMyKey);

	return SUCCESS;
}

/******************************************************************************
 *
 * DESCRIPTION:
 *
 *
 *
 ****************************************************************************/
int GetStatsFromRegistry(struct reg_stats* pStats, char *name)
{
	char 	buf[100];
	long	lResult;
	HKEY	hkeyMyKey;
	long	lDisposition;
	char 	keybuf[200];
	char 	fullbuf[200];
	long	lSize = sizeof(buf);
	long 	Type;


	GetRsyncKey(keybuf, sizeof(keybuf));
	sprintf(fullbuf, "%s\\%s", keybuf, name);
	//consoleprintf( "GetStatsFromRegistry %s\n", fullbuf);

	// Create or open the registry key
	lResult = RegCreateKeyEx(
					HKEY_LOCAL_MACHINE, 		// Key
					(BYTE*)fullbuf, 			// Sub-key
					0,								// Reserved
					NULL,							// Reserved
					REG_OPTION_NON_VOLATILE,// Really save in registry
					KEY_ALL_ACCESS,			// Allow all access
					0,								// Reserved
					&hkeyMyKey,					// Returns this handle
					&lDisposition);			// Was it created or just opened?

	if (lResult != 0)
	{
		// return error
		consoleprintf( "GetStatsFromRegistry RegCreateKey %d\n", lResult);
		return FAIL;
	}

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_STATUS,	// Name of the value to get
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->status = atof(buf);

	lSize = sizeof(pStats->error);
	pStats->error[0] = '\0';
	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_RSYNC_ERROR,	// Name of the value to get
					0, &Type,
					(BYTE*)pStats->error,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0 || pStats->error[0] == '\0')
	{
		lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
						(BYTE*)REGISTRYVALUE_ERRSTRING,	// Name of the value to get
						0, &Type,
						(BYTE*)pStats->error,			// Address of value data
						&lSize);// Size of value data include null
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);

	lSize = sizeof(buf);
	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_TOTALWRITE,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->total_written = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_TOTALREAD,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->total_read = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_TOTALSIZE,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->total_size = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_TOTAL_XFER_SIZE,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
	//consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->total_transferred_size = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_NUM_XFER_FILES,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf,"0");
	}
//	consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->num_transferred_files = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_FILELISTCOUNT,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf, "0");
	}
//	consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->num_files = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_FILELISTSIZE,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf, "0");
	}
//	consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->flist_size = atof(buf);

	lResult = RegQueryValueEx(hkeyMyKey,		// Handle to key to set value for
					(BYTE*)REGISTRYVALUE_FILELISTINDEX,				// Name of the value to set
					0, &Type,
					(BYTE*)buf,			// Address of value data
					&lSize);// Size of value data include null

	if (lResult != 0)
	{
		sprintf(buf, "0");
	}
//	consoleprintf( "GetStatsFromRegistry RegQueryValueEx %d %s\n", lResult, buf);
	pStats->flist_index = atof(buf);

//	if (SUCCESS != GetTimeFromRegistry(REGISTRYVALUE_STARTTIME, &start))
//		goto End;
//
//	if (SUCCESS != GetTimeFromRegistry(REGISTRYVALUE_TIME, &end))
//		goto End;
//
//	diff = difftime(end,start);
//	consoleprintf("SaveStats %d %d %d\n", start, end, diff);
//
//End:
	// Close the registry
	lResult = RegCloseKey(hkeyMyKey);

	return SUCCESS;
}

void ap_set_request(ap_sb_handle_t *sb_handle, const char* module_name, const char*request, const char *nwauth_user, const char *host, const char *addr)
{
	extern int am_daemon;

	struct reg_stats regStats;
	worker_score *ws;
	long regTime;
	char *name = NULL;

	if (!sb_handle)
		return;
	
	ws = ap_get_scoreboard_worker(sb_handle->child_num, sb_handle->thread_num);
	if (!ws)
		return;

	if (module_name)
		strlcpy(ws->module, module_name, sizeof ws->module);
		
	if (request)
		strlcpy(ws->request, request, sizeof ws->request);
	
	if (nwauth_user)
		strlcpy(ws->client, addr, sizeof ws->client);
		
	if (host)
		strlcpy(ws->hostname, host, sizeof ws->hostname);
		
	if (addr)
		strlcpy(ws->hostip, addr, sizeof ws->hostip);
	
	if (!am_daemon)
	{
		int i = 1;
		if (stricmp(ws->module,ADMIN) ==0)
			name = DATA;
		else
			name = ADMIN;
			
		if (sb_handle->thread_num == 0)
			i = 1;
		else 
			i = 0;
			
		if (SUCCESS == GetStatsFromRegistry(&regStats, name))
		{
			worker_score *ws =  ap_get_scoreboard_worker(0, i);
			memset(ws->module, 0, sizeof(ws->module));
			strncpy(ws->module, name, sizeof(ws->module)-1);
			memset(ws->error, 0, sizeof(ws->error));
			strncpy(ws->error, regStats.error, sizeof(ws->error)-1);

			ws->total_size = regStats.total_size;
			ws->total_transferred_size = regStats.total_transferred_size;
			ws->total_written = regStats.total_written;
			ws->total_read = regStats.total_read;
			ws->num_transferred_files = regStats.num_transferred_files;
			ws->num_files = regStats.num_files;
			ws->flist_index = regStats.flist_index;
			ws->flist_size = regStats.flist_size;
			ws->status = regStats.status;

			if (SUCCESS == GetSBTimeFromRegistry(REGISTRYVALUE_STARTTIME, &regTime, name))
			{
				ws->start_time = regTime;
			}
			if (SUCCESS == GetSBTimeFromRegistry(REGISTRYVALUE_TIME, &regTime, name))
			{
				ws->stop_time = regTime;
			}
//			ap_increment_process_stats(0, &regStats);
		}

	}	
//	consoleprintf( "request %s %s %s %s %s\n",
//			module_name ? module_name : "NULL",
//			request ? request : "NULL",
//			auth_user ? auth_user : "NULL",
//			addr ? addr : "NULL");			
}

int ap_update_child_status(ap_sb_handle_t *sbh, int status)
{
    return ap_update_child_status_from_indexes(sbh->child_num, sbh->thread_num,
                                               status);
}

ap_sb_handle_t * ap_create_sb_handle(int child_num, int threadid)
{
	ap_sb_handle_t * sb_handle;
	int y;

    if ((child_num < 0) || (server_limit < child_num))
        return(NULL); /* Out of range */
	
//	rprintf(FLOG, "scoreboard %x %x\n", ap_scoreboard_image, &ap_scoreboard_image->servers[0][0]);
//	consoleprintf( "ap_create_sb_handle %d %d\n", child_num, threadid);
	
	for( y = 0; y < thread_limit; y++)
	{
		worker_score * ws = &ap_scoreboard_image->servers[child_num][y];
//		consoleprintf( "worker %x %d\n", ws, ws->status);
		if (ws->status == SERVER_DEAD)
		{
			sb_handle = malloc(sizeof (ap_sb_handle_t));
			if (!sb_handle)
				return NULL;
				
			sb_handle->child_num = child_num;
		    sb_handle->thread_num = y;
//			consoleprintf( "ap_create_sb_handle %x %x %x\n", 
//				sb_handle, sb_handle->child_num, sb_handle->thread_num );
			return sb_handle;
		}
	}
	return NULL;
}

void ap_delete_sb_handle(ap_sb_handle_t * sb_handle)
{
	worker_score *ws;
//	consoleprintf("ap_delete_sb_handle %x\n", sb_handle);

	if (!sb_handle)
		return;

//	consoleprintf( "ap_delete_sb_handle %x %x %x\n", 
//		sb_handle, sb_handle->child_num, sb_handle->thread_num );
	
	ws = ap_get_scoreboard_worker(sb_handle->child_num, sb_handle->thread_num);
	if (!ws)
		return;
		
//	consoleprintf("ap_delete_sb_handle %x\n", ws);
	memset(ws, 0x00, sizeof (worker_score));
	
	free(sb_handle);
}

void ap_increment_counts(ap_sb_handle_t *sb, struct jni_stats *r)
{
    worker_score *ws;
	unsigned long 	end, diff;

	if (!sb)
		return;
    
	ws = ap_get_scoreboard_worker(sb->child_num, sb->thread_num);
	if (!ws)
	{
		rprintf(FLOG,"ap_increment_counts failed to get worker %x %x\n", sb->child_num, sb->thread_num);
		return;
	}

//	consoleprintf("ap_increment_counts %x\n", ws);
#ifdef HAVE_TIMES
    times(&ws->times);
#endif
//    ws->access_count++;
//    ws->my_access_count++;
//    ws->conn_count++;
//    ws->bytes_served += r->total_written;
//    ws->my_bytes_served += r->total_written;
//    ws->conn_bytes += r->total_transferred_size;
	ws->total_size = r->total_size;
	ws->total_transferred_size = r->total_transferred_size;
	ws->total_written = r->total_written;
	ws->total_read = r->total_read;
	ws->literal_data = r->literal_data;
	ws->matched_data = r->matched_data;
	ws->flist_size = r->flist_size;
	ws->num_files = r->num_files;
	ws->num_transferred_files = r->num_transferred_files;
//	ws->jni_starttime = r->jni_starttime;
	strlcpy(ws->op,r->op,(sizeof r->op));
	strlcpy(ws->fname,r->fname,SCOREMAXPATHLEN);
	ws->flist_index = r->flist_index;

	end = computeCurrentTime();
	diff = difftime(end,ws->start_time);
	if (diff != 0)
	{
		double thruput = (r->total_written+r->total_read)/(0.5 + diff);
		ws->thruput = ROUND(thruput);
	}
}

scoreboard * ap_get_scoreboard(void)
{
	return ap_scoreboard_image;
}

void ap_increment_process_counts(int process_num, int numConnections, int numThreads)
{
	process_score *ps =  ap_get_scoreboard_process(process_num);
	if (!ps)
		return;
		
	ps->connections += numConnections;                           
	ps->threads += numThreads;
//	consoleprintf("ap_increment_process_counts %x %x %x\n", ps, ps->connections, ps->threads);
}

void ap_decrement_process_counts(int process_num, int numConnections, int numThreads)
{
	process_score *ps =  ap_get_scoreboard_process(process_num);
	if (!ps)
		return;
		
	ps->connections -= numConnections;                           
	ps->threads -= numThreads;
//	consoleprintf("ap_decrement_process_counts %x %x %x\n", ps, ps->connections, ps->threads);
}

#undef stats
void ap_increment_process_stats(int process_num, struct stats* pStats)
{
	process_score *ps =  ap_get_scoreboard_process(process_num);
	if (!ps)
		return;
		
	ps->total_size += pStats->total_size;
	ps->total_transferred_size += pStats->total_transferred_size;
	ps->total_written += pStats->total_written;
	ps->total_read += pStats->total_read;
	ps->num_transferred_files += pStats->num_transferred_files;
}

scoreboard * ap_init_scoreboard_copy(void *shared_score)
{
	scoreboard *rv = NULL;
    char *more_storage;
    int i;
    
    ap_calc_scoreboard_size();
    rv = calloc(1, sizeof(scoreboard) + server_limit * sizeof(worker_score *));
    more_storage = shared_score;
    rv->global = (global_score *)more_storage;
    more_storage += sizeof(global_score);
    rv->parent = (process_score *)more_storage;
    more_storage += sizeof(process_score) * server_limit;
    rv->servers = 
        (worker_score **)((char*)rv + sizeof(scoreboard));
    for (i = 0; i < server_limit; i++) {
        rv->servers[i] = (worker_score *)more_storage;
        more_storage += thread_limit * sizeof(worker_score);
    }
    assert(more_storage == (char*)shared_score + scoreboard_size);
    rv->global->server_limit = server_limit;
    rv->global->thread_limit = thread_limit;
    return rv;
}

scoreboard * ap_get_scoreboard_copy(void)
{
	scoreboard *rv = NULL;

    int i;
#if APR_HAS_SHARED_MEMORY
    apr_status_t rv;
#endif
//	consoleprintf( "ap_get_scoreboard_copy %x %x\n", ap_scoreboard_image, &ap_scoreboard_image->servers[0][0]);

//	for( y = 0; y < thread_limit; y++)
//	{
//		worker_score * ws = &ap_scoreboard_image->servers[0][y];
//		if (ws->status)
//			consoleprintf( "ap_get_scoreboard_copy:live %x %d %d\n", ws, y, ws->status);
//	}
	
    if (ap_scoreboard_image) 
    {
	    ap_calc_scoreboard_size();
	    {
	        /* A simple malloc will suffice */
	        void *sb_mem = calloc(1, scoreboard_size);
	        if (sb_mem == NULL) {
	            consoleprintf(
	                         "(%d)%s: cannot allocate scoreboard",
	                         errno, strerror(errno));
	            return NULL;
	        }
	        rv = ap_init_scoreboard_copy(sb_mem);
	    }
        *rv->global = *ap_scoreboard_image->global;
        memcpy(rv->parent, ap_scoreboard_image->parent, 
               sizeof(process_score) * server_limit);
        for (i = 0; i < server_limit; i++) {
            memcpy(rv->servers[i], ap_scoreboard_image->servers[i],
                   sizeof(worker_score) * thread_limit);
        }
//		consoleprintf( "ap_get_scoreboard_copy %x %x\n", rv, &rv->servers[0][0]);
//		for( y = 0; y < thread_limit; y++)
//		{
//			worker_score * ws = &rv->servers[0][y];
//			if (ws->status)
//				consoleprintf( "ap_get_scoreboard_copy:copy %x %d %d\n", ws, y, ws->status);
//		}

        return rv;
    }
    
    return NULL;
}

char * longprintf(char *buf, int size, int64 value)
{
	snprintf(buf,size,"%.0f", (double)value);
	return buf;
}

void ap_display_scoreboard (void)
{
	scoreboard * pStats;
	int y;

	if (ap_scoreboard_image->global->sb_type == SB_DISPLAY)
	{   
		extern int gScoreHandle;
		char buf[256];

		SetCurrentScreen(gScoreHandle);
		pStats = ap_get_scoreboard();
		ClearScreen();
		printf("RSync Scoreboard\n");
//		printf("server_limit = %d\n", pStats->global->server_limit);
//		printf("thread_limit = %d\n", pStats->global->thread_limit);
//		printf("sb_type = %d\n", pStats->global->sb_type);
//		printf("running_generation = %d\n", pStats->global->running_generation);
		printf("restart_time = %s\n", timestring(pStats->global->restart_time));
		printf("connections = %ld\n", pStats->parent->connections);
		printf("threads = %ld\n", pStats->parent->threads);
		printf("total_size = %s\n", longprintf(buf, sizeof(buf), pStats->parent->total_size));
		printf("total_transferred_size = %s\n", longprintf(buf, sizeof(buf), pStats->parent->total_transferred_size));
		printf("total_written = %s\n", longprintf(buf, sizeof(buf), pStats->parent->total_written));
		printf("total_read = %s\n", longprintf(buf, sizeof(buf), pStats->parent->total_read));
		printf("num_transferred_files = %s\n", longprintf(buf, sizeof(buf), pStats->parent->num_transferred_files));

		for(y = 0; y < pStats->global->thread_limit; y++)
		{
			worker_score * ws = ap_get_scoreboard_worker(0, y);
	//				printf("worker = %x\n", ws);
			if (ws->status != 0)
			{
				printf("\ttotal_size = %s\n", longprintf(buf, sizeof(buf), ws->total_size));
				printf("\ttotal_transferred_size = %s\n", longprintf(buf, sizeof(buf), ws->total_transferred_size));
				printf("\ttotal_written = %s\n", longprintf(buf, sizeof(buf), ws->total_written));
				printf("\ttotal_read = %s\n", longprintf(buf, sizeof(buf), ws->total_read));
				printf("\tliteral_data = %s\n", longprintf(buf, sizeof(buf), ws->literal_data));
				printf("\tmatched_data = %s\n", longprintf(buf, sizeof(buf), ws->matched_data));
				printf("\tflist_size = %s\n", longprintf(buf, sizeof(buf), ws->flist_size));
				printf("\tnum_files = %s\n", longprintf(buf, sizeof(buf), ws->num_files));
				printf("\tnum_transferred_files = %s\n", longprintf(buf, sizeof(buf), ws->num_transferred_files));
				printf("\tstart time = %s\n", timestring(ws->start_time));
				printf("\top = %s\n", ws->op);
				printf("\tfname = %s\n", ws->fname);
				printf("\tflist_index = %d\n", longprintf(buf, sizeof(buf), ws->flist_index));
				printf("\tmodule = %s\n", ws->module ? ws->module : "NULL");
				printf("\tclient = %s\n", ws->client ? ws->client : "NULL");
				printf("\trequest = %s\n", ws->request ? ws->request : "NULL");
				printf("\thost name = %s\n", ws->hostname ? ws->hostname : "NULL");
				printf("\thost ip = %s\n\n", ws->hostip ? ws->hostip : "NULL");
			}
		}
	   SetCurrentScreen(gScreenHandle);
	}
}
