#######################################################################
#
# LINKRULE.MAK
#
#######################################################################

!IFDEF MAKE_MESSAGES
# Only make messages if explicitly requested. This is so Engineers can
# build the messages when changes occur, check them in, and they will
# not need rebuilding. Otherwise, the MDBs and MSGs keep getting
# rebuilt in the PREP areas causing messages and MDBs to change out of
# control and cause problems determining which versions need
# translating.

#***********************************************************************
# If the MESSAGES macro is defined, ensure we update all the message
# files.
#***********************************************************************
!IFDEF MESSAGES

#***********************************************************************
# Update the Message database from any code files (.mlh/mlc)
#***********************************************************************
$(MDB_DIR)\$(TARGET).mdb:           $(MDB_DIR)\$(TARGET).mlc

#***********************************************************************
# Make the .msg files from the message database
#***********************************************************************
$(MESSAGE_DIR)\$(TARGET).msg:       $(MDB_DIR)\$(TARGET).mdb

$(WIDEJP_DIR)\$(TARGET).MSG:        $(MDB_DIR)\$(TARGET).mdb

$(EUROPE_DIR)\$(TARGET).MSG:        $(MDB_DIR)\$(TARGET).mdb

$(WORD_COUNT_DIR)\$(TARGET).CNT:    $(MDB_DIR)\$(TARGET).mdb

fake_messages:                      $(WIDEJP_DIR)\$(TARGET).MSG \
                                    $(EUROPE_DIR)\$(TARGET).MSG \
                                    $(WORD_COUNT_DIR)\$(TARGET).CNT
!IFDEF CCM_MAKE_DEF_SERIAL
    # At this point, all message files will have been updated if needed.
	# We can now remove the is_product attribute so that developers can
	# check this file in.
    -@echo Removing PRODUCT attribute from Continuus message files
    # Ignoring result, since it may fail if there is no product
	# attribute or if the targets are not writable because they didn't
	# need rebuilding.
    -$(SILENT_CMD)ccm attr -d is_product $(MDB_DIR)\$(TARGET).mdb
    -$(SILENT_CMD)ccm attr -d is_product $(MESSAGE_DIR)\$(TARGET).msg
    -$(SILENT_CMD)ccm attr -d is_product $(WIDEJP_DIR)\$(TARGET).msg
    -$(SILENT_CMD)ccm attr -d is_product $(EUROPE_DIR)\$(TARGET).msg
    -$(SILENT_CMD)ccm attr -d is_product $(WORD_COUNT_DIR)\$(TARGET).cnt
!ELSE
    -@echo .
!ENDIF

MESSAGE_DEPENDANCIES=fake_messages

#Endif MESSAGES
!ENDIF

#Endif MAKE_MESSAGES
!ENDIF

#***********************************************************************
# The default target is $(TARGET) which will do a compile and link.
#***********************************************************************
$(TARGET):  compile $(TARGET).MAP $(TARGET).NLM $(MESSAGE_DEPENDANCIES)

#***********************************************************************
# Define a rule for making a .MAP file. It really is not a rule,
# but it is enough to make continuus check out the file and make it
# RW as it is a has the PRODUCT attribute set.
#***********************************************************************
$(TARGET).MAP:                      $(DEPENDANCIES) .\builtins.mak .\linkrule.mak
!IFDEF CCM_MAKE_DEF_SERIAL
    -@echo Making Continuus product file $(TARGET).MAP writable
!ELSE
    -@echo .
!ENDIF

#***********************************************************************
# Define how to make an NLM based on all the predefined macros.
#***********************************************************************
$(TARGET).NLM: $(DEPENDANCIES) .\builtins.mak .\linkrule.mak
!IF "$(CONTINUE)" == "-"
	$(SILENT_CMD)checkerr
!ENDIF
    -@echo Linking $(NLM)
!IFDEF MESSAGES
    -$(SILENT_CMD)attrib -r $(MESSAGES)
!ENDIF
!IFDEF USE_WLINK
    -$(SILENT_SET)set WATCOM=$(WATCOM)
    $(WLINK_CMD) @<<$(*R).lnk
!IFDEF VERBOSE
        OPTION VERBOSE
!ELSE
        OPTION QUIET
!ENDIF
!IFNDEF TYPE
        FORMAT  NOVELL NLM '$(PROG_DESC)'
!ELSE
!IF $(TYPE) = "0"
        FORMAT  NOVELL NLM '$(PROG_DESC)'
!ENDIF
!IF $(TYPE) = "1"
        FORMAT  NOVELL LAN '$(PROG_DESC)'
!ENDIF
!IF $(TYPE) = "2"
        FORMAT  NOVELL DSK '$(PROG_DESC)'
!ENDIF
!IF $(TYPE) = "3"
        FORMAT  NOVELL NAM '$(PROG_DESC)'
!ENDIF
!ENDIF
        NAME    $*
!IFNDEF PRODUCTION
        DEBUG   CODEVIEW
        DEBUG   NOVELL
!IFDEF RDEBUG
        OPTION SYMFILE=$(TARGET).sym
!ENDIF
!ELSE
!IFDEF PRODUCTION_SYMBOLS
        DEBUG   CODEVIEW
        DEBUG   NOVELL
!ENDIF
!ENDIF
!IFDEF OBJECT_FILES
        FILE    $(OBJECT_FILES)
!ENDIF
!IFDEF OBJECT_FILES_2
        FILE    $(OBJECT_FILES_2)
!ENDIF
!IFDEF OBJECT_FILES_3
        FILE    $(OBJECT_FILES_3)
!ENDIF
!IFDEF MODULE_DEPENDANCIES
        MODULE  $(MODULE_DEPENDANCIES)
!ENDIF
!IFDEF COPYRIGHT
        OPTION  COPYRIGHT '$(COPYRIGHT)'
!ENDIF
        OPTION  VERSION=$(MAJOR_VERSION).$(MINOR_VERSION).$(REVISION)
!IFDEF THREADNAME
        OPTION  THREADNAME '$(THREADNAME)'
!ENDIF
!IFDEF SYNCHRONIZE
        OPTION  SYNCHRONIZE
!ENDIF
!IFDEF REENTRANT
        OPTION REENTRANT
!ENDIF
!IFDEF MULTILOAD
        OPTION MULTILOAD
!ENDIF
        OPTION  MAP=$(TARGET).map
!IFDEF SCREENNAME
        OPTION  SCREENNAME '$(SCREENNAME)'
!ENDIF
!IFDEF XDC_FILE
        OPTION  XDCDATA=$(XDC_FILE)
!ENDIF
!IFDEF START_FUNCTION
        OPTION  START=$(START_FUNCTION)
!ENDIF
!IFDEF EXIT_FUNCTION
        OPTION  EXIT=$(EXIT_FUNCTION)
!ENDIF
!IFDEF CHECK_FUNCTION
        OPTION  CHECK=$(CHECK_FUNCTION)
!ENDIF
!IFDEF PSEUDOPREEMPTION
        OPTION  PSEUDOPREEMPTION
!ENDIF
!IFDEF STACK
        OPTION  STACK=$(STACK)
!ENDIF
!IFDEF MESSAGES
        OPTION  MESSAGES=$(MESSAGES)
!ENDIF
!IFDEF HELPFILE
        OPTION  HELP=$(HELPFILE)
!ENDIF
!IFDEF FLAG_ON
        OPTION  NLMFLAGS=$(FLAG_ON)
!ENDIF
!IFDEF EXPORT_FILES
        EXPORT  $(EXPORT_FILES)
!ENDIF

!IFNDEF PRODUCTION
!IFDEF EXPORT_FILES_DEBUG
export      $(EXPORT_FILES_DEBUG)
!ENDIF
!ENDIF

!IFDEF EXPORT_FUNCTIONS
        EXPORT  $(EXPORT_FUNCTIONS)
!ENDIF
!IFDEF IMPORT_FILES
        IMPORT  $(IMPORT_FILES)
!ENDIF
!IFDEF IMPORT_FUNCTIONS
        IMPORT  $(IMPORT_FUNCTIONS)
!ENDIF
!IFDEF LIBRARY_FILES
        LIBRARY  $(LIBRARY_FILES)
!ENDIF
OPTION  NODEFAULTLIBS
OPTION  CASEEXACT
# Do not use "option eliminate", since it can remove NLMCheck or other necessary
# but internally unreferenced code.
# OPTION  ELIMINATE
#OPTION  MANGLEDNAMES
option manglednames
		OPTION  VERBOSE
!IFNDEF NO_ALIAS
alias InternalDiskGetNoReadNoWaitNoAl=InternalDiskGetNoReadNoWaitNoAlloc
!ENDIF
!IFDEF KEEP_FILES
<<KEEP
!ELSE
<<NOKEEP
!ENDIF
!ELSE
    $(LINKER_CMD) <<$(*R).lnk
Description "$(PROG_DESC)"
Output $@
!IFDEF  TYPE
TYPE        $(TYPE)
!ELSE
Type        0
!ENDIF
Copyright   "$(COPYRIGHT)"
Version     $(MAJOR_VERSION),$(MINOR_VERSION),$(REVISION)
!IFDEF OBJECT_FILES
Input       $(OBJECT_FILES)
!ENDIF
!IFDEF OBJECT_FILES_2
Input       $(OBJECT_FILES_2)
!ENDIF
!IFDEF OBJECT_FILES_3
Input       $(OBJECT_FILES_3)
!ENDIF
!IFDEF MESSAGES
messages    $(MESSAGES)
!ENDIF
!IFDEF HELPFILE
help        $(HELPFILE)
!ENDIF
!IFNDEF PRODUCTION
debug
!ELSE
!IFDEF PRODUCTION_SYMBOLS
debug
!ENDIF
!ENDIF
Map         $(TARGET).MAP
!IFDEF SCREENNAME
Screenname  "$(SCREENNAME)"
!ENDIF
!IFDEF THREADNAME
Threadname  "$(THREADNAME)"
!ENDIF
!IFDEF SYNCHRONIZE
Synchronize
!ENDIF
!IFDEF REENTRANT
reentrant
!ENDIF
!IFDEF MULTILOAD
multiload
!ENDIF
!IFDEF PSEUDOPREEMPTION
PseudoPreemption
!ENDIF
!IFDEF STACK
stack       $(STACK)
!ENDIF
!IFDEF START_FUNCTION
start       $(START_FUNCTION)
!ENDIF
!IFDEF CHECK_FUNCTION
check       $(CHECK_FUNCTION)
!ENDIF
!IFDEF EXIT_FUNCTION
exit        $(EXIT_FUNCTION)
!ENDIF
!IFDEF FLAG_ON
flag_on     $(FLAG_ON)
!ENDIF
!IFDEF IMPORT_FILES
import      $(IMPORT_FILES)
!ENDIF
!IFDEF IMPORT_FUNCTIONS
import      $(IMPORT_FUNCTIONS)
!ENDIF
!IFDEF EXPORT_FILES
export      $(EXPORT_FILES)
!ENDIF

!IFNDEF PRODUCTION
!IFDEF EXPORT_FILES_DEBUG
export      $(EXPORT_FILES_DEBUG)
!ENDIF
!ENDIF

!IFDEF EXPORT_FUNCTIONS
export      $(EXPORT_FUNCTIONS)
!ENDIF
!IFDEF MODULE_DEPENDANCIES
module      $(MODULE_DEPENDANCIES)
!ENDIF
!IFDEF LIBRARY_FILES
        LIBRARY  $(LIBRARY_FILES)
!ENDIF
!IFDEF KEEP_FILES
<<KEEP
!ELSE
<<NOKEEP
!ENDIF
!ENDIF
!IFDEF RDEBUG
	@echo Packing the Symbol file (for RDEBUG)
	@cvpackz $(TARGET).sym >nul
!ENDIF
!IFNDEF NO_PACK
	@echo Packing $(TARGET).NLM
	@NLMPACKX $(TARGET).NLM $(TARGET).PAK >nul
	@del $(TARGET).NLM
	@rename $(TARGET).PAK $(TARGET).NLM
!ENDIF
!IFDEF CCM_MAKE_DEF_SERIAL
    -@echo Updating Continuus with new $(TARGET).MAP file
    ccm resync $(TARGET).MAP
!ENDIF

